{% extends "layouts/default/index.html" %}
{% block body %}
    <h1>Welcome to Django Frontend Kit</h1>
    <h2>This is a sample page</h2>
    <p>Here's a random number for you: {{ page.lucky_number }}</p>
{% endblock body %}
